/* BdfServer - Copyright (c) 2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.tools.instruction;

import fr.exemole.bdfserver.api.instruction.BdfInstructionException;
import fr.exemole.bdfserver.api.instruction.BdfParameters;
import net.fichotheque.addenda.Addenda;
import net.fichotheque.addenda.Document;
import net.fichotheque.album.Album;
import net.fichotheque.album.Illustration;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.corpus.FicheMeta;
import net.fichotheque.sphere.Redacteur;
import net.fichotheque.sphere.Sphere;
import net.fichotheque.thesaurus.Motcle;
import net.fichotheque.thesaurus.Thesaurus;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.text.RelativePath;


/**
 *
 * @author Vincent Calame
 */
public interface BdfInstructionMethods extends BdfParameters {

    public RequestMap getRequestMap();

    public default boolean checkConfirmation() {
        return BdfInstructionUtils.checkConfirmation(getRequestMap(), getBdfServer());
    }

    public default String getMandatory(String paramName) throws BdfInstructionException {
        return BdfInstructionUtils.getMandatoryParameter(getRequestMap(), paramName);
    }

    public default int getMandatoryInteger(String paramName) throws BdfInstructionException {
        return BdfInstructionUtils.getMandatoryIntegerParameter(getRequestMap(), paramName);
    }

    public default int getMandatoryId() throws BdfInstructionException {
        return BdfInstructionUtils.getMandatoryId(getRequestMap());
    }

    public default Corpus getMandatoryCorpus() throws BdfInstructionException {
        return BdfInstructionUtils.getMandatoryCorpus(getFichotheque(), getRequestMap());
    }

    public default Thesaurus getMandatoryThesaurus() throws BdfInstructionException {
        return BdfInstructionUtils.getMandatoryThesaurus(getFichotheque(), getRequestMap());
    }

    public default Sphere getMandatorySphere() throws BdfInstructionException {
        return BdfInstructionUtils.getMandatorySphere(getFichotheque(), getRequestMap());
    }

    public default Album getMandatoryAlbum() throws BdfInstructionException {
        return BdfInstructionUtils.getMandatoryAlbum(getFichotheque(), getRequestMap());
    }

    public default Addenda getMandatoryAddenda() throws BdfInstructionException {
        return BdfInstructionUtils.getMandatoryAddenda(getFichotheque(), getRequestMap());
    }

    public default FicheMeta getMandatoryFicheMeta() throws BdfInstructionException {
        return BdfInstructionUtils.getMandatoryFicheMeta(getFichotheque(), getRequestMap());
    }

    public default Motcle getMandatoryMotcle() throws BdfInstructionException {
        return BdfInstructionUtils.getMandatoryMotcle(getFichotheque(), getRequestMap());
    }

    public default Redacteur getMandatoryRedacteur() throws BdfInstructionException {
        return BdfInstructionUtils.getMandatoryRedacteur(getFichotheque(), getRequestMap());
    }

    public default Document getMandatoryDocument() throws BdfInstructionException {
        return BdfInstructionUtils.getMandatoryDocument(getFichotheque(), getRequestMap());
    }

    public default Illustration getMandatoryIllustration() throws BdfInstructionException {
        return BdfInstructionUtils.getMandatoryIllustration(getFichotheque(), getRequestMap());
    }

    public default RelativePath getMandatoryPath() throws BdfInstructionException {
        return BdfInstructionUtils.getMandatoryResourcePath(getRequestMap());
    }


}
