/* BdfServer - Copyright (c) 2015-2022 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.tools.roles;

import fr.exemole.bdfserver.api.roles.PermissionSummary;
import java.util.function.Predicate;
import net.fichotheque.SubsetItem;
import net.fichotheque.corpus.FicheMeta;
import net.fichotheque.utils.SphereUtils;
import net.mapeadores.util.exceptions.SwitchException;


/**
 *
 * @author Vincent Calame
 */
public class PermissionPredicate implements Predicate<SubsetItem> {

    private final PermissionSummary permissionSummary;
    private final boolean read;


    private PermissionPredicate(PermissionSummary permissionSummary, boolean read) {
        this.permissionSummary = permissionSummary;
        this.read = read;
    }


    @Override
    public boolean test(SubsetItem subsetItem) {
        int level;
        if (read) {
            level = permissionSummary.getReadLevel(subsetItem.getSubsetKey());
        } else {
            level = permissionSummary.getWriteLevel(subsetItem.getSubsetKey());
        }
        switch (level) {
            case PermissionSummary.SUMMARYLEVEL_0_NONE:
                return false;
            case PermissionSummary.SUMMARYLEVEL_1_FICHE_OWN:
                return SphereUtils.ownsToRedacteur((FicheMeta) subsetItem, permissionSummary.getRedacteur());
            case PermissionSummary.SUMMARYLEVEL_2_FICHE_SPHERE:
                return SphereUtils.ownsToSphere((FicheMeta) subsetItem, permissionSummary.getRedacteur().getSphere());
            case PermissionSummary.SUMMARYLEVEL_3_CROISEMENT_TEST:
                return RoleUtils.croisementTest(subsetItem, permissionSummary, read);
            case PermissionSummary.SUMMARYLEVEL_4_ALL:
            case PermissionSummary.SUMMARYLEVEL_5_ADMIN:
                return true;
            default:
                throw new SwitchException("Unknown level: " + level);
        }
    }

    public static PermissionPredicate read(PermissionSummary permissionSummary) {
        return new PermissionPredicate(permissionSummary, true);
    }

    public static PermissionPredicate write(PermissionSummary permissionSummary) {
        return new PermissionPredicate(permissionSummary, false);
    }

}
