/* BdfServer - Copyright (c) 2015-2022 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.tools.ui.components;

import fr.exemole.bdfserver.api.ui.FieldUi;
import fr.exemole.bdfserver.api.ui.options.InputOption;
import fr.exemole.bdfserver.api.ui.options.ValueOption;
import fr.exemole.bdfserver.tools.ui.options.UiOptionsUtils;
import net.fichotheque.corpus.metadata.FieldKey;
import net.mapeadores.util.annotation.Nullable;
import net.mapeadores.util.attr.AttributeChange;
import net.mapeadores.util.attr.Attributes;
import net.mapeadores.util.attr.AttributesBuilder;


/**
 *
 * @author Vincent Calame
 */
public class FieldUiBuilder {

    private final FieldKey fieldKey;
    private final AttributesBuilder attributesBuilder;
    private InputOption inputOption;
    private ValueOption valueOption;

    private FieldUiBuilder(FieldKey fieldKey, @Nullable Attributes initAttributes) {
        if (fieldKey == null) {
            throw new IllegalArgumentException("corpusField is null");
        }
        this.fieldKey = fieldKey;
        this.inputOption = UiOptionsUtils.getDefaultInputOption(fieldKey);
        this.valueOption = UiOptionsUtils.getDefaultValueOption(fieldKey);
        this.attributesBuilder = new AttributesBuilder(initAttributes);
    }

    public FieldUiBuilder setInputOption(InputOption inputOption) {
        if (this.inputOption != null) {
            this.inputOption = inputOption;
        }
        return this;
    }

    public FieldUiBuilder setValueOption(ValueOption valueOption) {
        if (this.valueOption != null) {
            this.valueOption = valueOption;
        }
        return this;
    }

    public FieldUiBuilder changesAttribute(AttributeChange attributeChange) {
        this.attributesBuilder.changeAttributes(attributeChange);
        return this;
    }

    public AttributesBuilder getAttributesBuilder() {
        return attributesBuilder;
    }

    public FieldUi toFieldUi() {
        return new InternalFieldUi(fieldKey, inputOption, valueOption, attributesBuilder.toAttributes());
    }

    public static FieldUiBuilder init(FieldKey fieldKey) {
        return new FieldUiBuilder(fieldKey, null);
    }

    public static FieldUiBuilder init(FieldKey fieldKey, @Nullable Attributes initAttributes) {
        return new FieldUiBuilder(fieldKey, initAttributes);
    }

    public static FieldUiBuilder derive(FieldUi fieldUi) {
        return FieldUiBuilder.init(fieldUi.getFieldKey(), fieldUi.getAttributes())
                .setInputOption(fieldUi.getInputOption())
                .setValueOption(fieldUi.getValueOption());
    }


    private static class InternalFieldUi implements FieldUi {

        private final FieldKey fieldKey;
        private final InputOption inputOption;
        private final ValueOption valueOption;
        private final Attributes attributes;

        private InternalFieldUi(FieldKey fieldKey, InputOption inputOption, ValueOption valueOption, Attributes attributes) {
            this.fieldKey = fieldKey;
            this.inputOption = inputOption;
            this.valueOption = valueOption;
            this.attributes = attributes;
        }

        @Override
        public FieldKey getFieldKey() {
            return fieldKey;
        }

        @Override
        public InputOption getInputOption() {
            return inputOption;
        }

        @Override
        public ValueOption getValueOption() {
            return valueOption;
        }

        @Override
        public String getName() {
            return fieldKey.getFieldString();
        }

        @Override
        public Attributes getAttributes() {
            return attributes;
        }

    }

}
