/* BdfServer_Commands - Copyright (c) 2011-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands.corpus;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.BdfServerEditor;
import fr.exemole.bdfserver.api.instruction.BdfInstructionException;
import fr.exemole.bdfserver.api.interaction.Domains;
import fr.exemole.bdfserver.tools.ui.UiUtils;
import net.fichotheque.ExistingSubsetException;
import net.fichotheque.SubsetKey;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.corpus.CorpusEditor;
import net.fichotheque.tools.corpus.CorpusTools;
import net.mapeadores.util.exceptions.ShouldNotOccurException;
import net.mapeadores.util.request.RequestMap;


/**
 *
 * @author Vincent Calame
 */
public class CloneCommand extends AbstractCorpusCreationCommand {

    public final static String COMMANDNAME = "Clone";
    public final static String COMMANDKEY = "_ CRP-21";
    public static String NEWCORPUS_PARAMNAME = "newcorpus";
    private Corpus corpus;
    private SubsetKey newCorpusKey;
    private CorpusEditor newCorpusEditor;

    public CloneCommand(BdfServer bdfServer, RequestMap requestMap) {
        super(bdfServer, requestMap);
    }

    @Override
    public boolean needSynchronisation() {
        return true;
    }

    @Override
    protected void doCommand() {
        initEditSession(Domains.CORPUS + "/" + COMMANDNAME);
        bdfServer.bufferFichothequeListener();
        try {
            newCorpusEditor = editSession.getFichothequeEditor().createCorpus(newCorpusKey, masterSubset);
        } catch (ExistingSubsetException ese) {
            throw new ShouldNotOccurException("test before ExistingSubsetException");
        }
        CorpusTools.copy(corpus.getCorpusMetadata(), newCorpusEditor.getCorpusMetadataEditor());
        bdfServer.flushFichothequeListener();
        BdfServerEditor bdfServerEditor = bdfServer.getBdfServerEditor();
        UiUtils.copyUi(bdfServerEditor, corpus, newCorpusEditor.getCorpus());
        bdfServerEditor.saveChanges();
        putResultObject(CORPUS_OBJ, newCorpusEditor.getCorpus());
        setDone("_ done.corpus.clone", corpus.getSubsetKey().toString(), newCorpusKey.toString());
    }

    @Override
    protected boolean checkParameters() throws BdfInstructionException {
        checkGlobalAdmin();
        corpus = getMandatoryCorpus();
        String newCorpusName = getMandatory(NEWCORPUS_PARAMNAME);
        initSubsetMaster();
        if (newCorpusName.length() == 0) {
            setError("_ error.empty.corpusname");
            return false;
        }
        try {
            newCorpusKey = SubsetKey.parse(SubsetKey.CATEGORY_CORPUS, newCorpusName);
            if (bdfServer.getFichotheque().getSubset(newCorpusKey) != null) {
                setError("_ error.existing.corpus", newCorpusKey.toString());
                return false;
            }
        } catch (java.text.ParseException pe) {
            setError("_ error.wrong.corpusname", newCorpusName);
            return false;
        }
        return true;
    }

}
