/* BdfServer_Commands - Copyright (c) 2015-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands.corpus;

import fr.exemole.bdfserver.api.BdfServer;
import static fr.exemole.bdfserver.api.instruction.BdfInstructionConstants.CORPUS_OBJ;
import fr.exemole.bdfserver.api.instruction.BdfInstructionException;
import fr.exemole.bdfserver.api.interaction.Domains;
import fr.exemole.bdfserver.tools.instruction.AbstractBdfCommand;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.corpus.metadata.CorpusField;
import net.fichotheque.corpus.metadata.CorpusMetadataEditor;
import net.fichotheque.corpus.metadata.ExistingFieldKeyException;
import net.fichotheque.corpus.metadata.FieldKey;
import net.mapeadores.util.exceptions.ShouldNotOccurException;
import net.mapeadores.util.request.RequestMap;


/**
 *
 * @author Vincent Calame
 */
public class SoustitreActivationCommand extends AbstractBdfCommand {

    public final static String COMMANDNAME = "SoustitreActivation";
    public final static String COMMANDKEY = "_ CRP-02";
    private Corpus corpus;

    public SoustitreActivationCommand(BdfServer bdfServer, RequestMap requestMap) {
        super(bdfServer, requestMap);
    }

    @Override
    public boolean needSynchronisation() {
        return true;
    }

    @Override
    protected void doCommand() {
        initEditSession(Domains.CORPUS + "/" + COMMANDNAME);
        putResultObject(CORPUS_OBJ, corpus);
        CorpusMetadataEditor corpusMetadataEditor = editSession.getFichothequeEditor().getCorpusEditor(corpus.getSubsetKey()).getCorpusMetadataEditor();
        try {
            corpusMetadataEditor.createCorpusField(FieldKey.SOUSTITRE, CorpusField.NO_FICHEITEM_FIELD);
            setDone("_ done.corpus.soustitreactivation");
        } catch (ExistingFieldKeyException efke) {
            throw new ShouldNotOccurException("test done before");
        }
    }

    @Override
    protected boolean checkParameters() throws BdfInstructionException {
        corpus = getMandatoryCorpus();
        checkSubsetAdmin(corpus);
        if (corpus.getCorpusMetadata().getCorpusField(FieldKey.SOUSTITRE) != null) {
            setError("_ error.existing.fieldkey_soustitre");
            return false;
        }
        return true;
    }

}
