/* BdfServer_HtmlProducers - Copyright (c) 2012-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.htmlproducers.misc;

import fr.exemole.bdfserver.api.exportation.table.TableExportParameters;
import fr.exemole.bdfserver.api.interaction.Domains;
import fr.exemole.bdfserver.api.interaction.GetConstants;
import fr.exemole.bdfserver.api.interaction.InteractionConstants;
import fr.exemole.bdfserver.api.interaction.domains.MiscDomain;
import fr.exemole.bdfserver.api.managers.TableExportManager;
import fr.exemole.bdfserver.html.BdfServerHtmlProducer;
import fr.exemole.bdfserver.html.consumers.Button;
import fr.exemole.bdfserver.html.consumers.Grid;
import fr.exemole.bdfserver.html.consumers.PageUnit;
import fr.exemole.bdfserver.html.consumers.attributes.Deploy;
import fr.exemole.bdfserver.html.forms.TableExportFormHtml;
import fr.exemole.bdfserver.html.jslib.CommonJsLibs;
import fr.exemole.bdfserver.tools.users.BdfUserUtils;
import java.util.List;
import net.fichotheque.exportation.table.TableExportDescription;
import net.mapeadores.util.html.HA;
import net.mapeadores.util.text.Label;
import net.mapeadores.util.text.LabelUtils;
import fr.exemole.bdfserver.api.instruction.BdfParameters;


/**
 *
 * @author Vincent Calame
 */
public class TableExportFormHtmlProducer extends BdfServerHtmlProducer {

    private final TableExportManager tableExportManager;
    private final TableExportParameters tableExportParameters;

    public TableExportFormHtmlProducer(BdfParameters bdfParameters) {
        super(bdfParameters);
        this.tableExportManager = bdfServer.getTableExportManager();
        this.tableExportParameters = BdfUserUtils.getDefaultTableExportParameters(bdfUser);
        addJsLib(CommonJsLibs.DEPLOY);
        addJsLib(CommonJsLibs.FORMSTORAGE);
        setMainStorageKey(Domains.MISC, MiscDomain.TABLEEXPORTFORM_PAGE);
        addThemeCss("misc.css");
    }

    @Override
    public void printHtml() {
        start();
        this
                .__(PageUnit.start("action-Ods", "_ title.tableexport.stream"))
                .FORM_get(HA.action(GetConstants.TABLES_ROOT + "/" + GetConstants.GET_MULTI).attr("data-formstorage-key", "tableExportForm"))
                .__(printTableExportChoice())
                .__(TableExportFormHtml.HEADERTYPE_TITLE)
                .__(TableExportFormHtml.printHeaderTypeRadios(this, tableExportParameters.getHeaderType()))
                .__(printSpreadMerge())
                .__(Button.COMMAND, Button.submit("action-Ods", "_ submit.tableexport.stream_ods"))
                ._FORM()
                .__(PageUnit.END);
        end();
    }

    private boolean printTableExportChoice() {
        String defaultDivId = generateId();
        String nameDivId = generateId();
        List<TableExportDescription> tableExportDescriptionList = tableExportManager.getValidTableExportDescriptionList();
        if (!tableExportDescriptionList.isEmpty()) {
            String tableExportName = tableExportParameters.getTableExportName();
            boolean isDefault = (tableExportName == null);
            this
                    .__(Grid.START)
                    .__(Grid.radioRow("_ label.tableexport.tableexport_name", name(GetConstants.DEFAULTEXPORT_PARAMNAME).value("0").checked(!isDefault).populate(Deploy.radio(nameDivId)),
                            () -> {
                                this
                                        .DIV(HA.id(nameDivId).classes("global-DetailPanel grid-Table").addClass(isDefault, "hidden"))
                                        .__(Grid.selectRow("_ label.global.tableexport", name(InteractionConstants.TABLEEXPORT_PARAMNAME),
                                                () -> {
                                                    for (TableExportDescription current : tableExportDescriptionList) {
                                                        String currentName = current.getName();
                                                        boolean selected = false;
                                                        if ((tableExportName != null) && (currentName.equals(tableExportName))) {
                                                            selected = true;
                                                        }
                                                        String title = "";
                                                        Label label = LabelUtils.getLangPartCheckedLabel(current.getTableExportDef().getTitleLabels(), workingLang);
                                                        if (label != null) {
                                                            title = " " + label.getLabelString();
                                                        }
                                                        this
                                                                .OPTION(currentName, selected)
                                                                .__escape("[")
                                                                .__escape(currentName)
                                                                .__escape("]")
                                                                .__escape(title)
                                                                ._OPTION();
                                                    }
                                                }))
                                        .__(Grid.checkboxRow("_ label.tableexport.withthesaurustable", name(GetConstants.THESAURUSTABLE_PARAMNAME).value("1")))
                                        ._DIV();
                            }))
                    .__(Grid.radioRow("_ label.tableexport.tableexport_default", name(GetConstants.DEFAULTEXPORT_PARAMNAME).value("1").checked(isDefault).populate(Deploy.radio(defaultDivId)),
                            () -> {
                                this
                                        .DIV(HA.id(defaultDivId).classes("global-DetailPanel").addClass(!isDefault, "hidden"))
                                        .__(TableExportFormHtml.printWithParameters(this, tableExportParameters.getDefaulFicheTableParameters()))
                                        .__(TableExportFormHtml.printPatternModeRadios(this, tableExportParameters.getDefaulFicheTableParameters().getPatternMode()))
                                        ._DIV();
                            }))
                    .__(Grid.END);
        } else {
            this
                    .__(TableExportFormHtml.printWithParameters(this, tableExportParameters.getDefaulFicheTableParameters()))
                    .__(TableExportFormHtml.printPatternModeRadios(this, tableExportParameters.getDefaulFicheTableParameters().getPatternMode()));
        }
        return true;
    }

    private boolean printSpreadMerge() {
        this
                .__(Grid.START)
                .__(Grid.checkboxRow("_ label.tableexport.spreadsheetmerge", name(GetConstants.MERGE_PARAMNAME).value("1")))
                .__(Grid.END);
        return true;
    }

}
