/* BdfServer_Multi - Copyright (c) 2022 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.multi.htmlproducers;

import fr.exemole.bdfserver.html.AppInit;
import fr.exemole.bdfserver.multi.Multi;
import fr.exemole.bdfserver.tools.BdfServerUtils;
import fr.exemole.bdfserver.tools.apps.AppConf;
import fr.exemole.bdfserver.tools.storage.StorageUtils;
import net.mapeadores.util.html.JsObject;
import net.mapeadores.util.html.TrustedHtml;
import net.mapeadores.util.html.jsoup.TrustedHtmlFactories;
import net.mapeadores.util.io.DocStream;
import net.mapeadores.util.text.RelativePath;


/**
 *
 * @author Vincent Calame
 */
public class AppHtmlProducer extends MultiHtmlProducer {

    private final AppConf appConf;

    public AppHtmlProducer(Multi multi, AppConf appConf) {
        super(multi);
        this.appConf = appConf;
        AppInit.init(this, appConf, multi.getWebappsResourceStorages(), null);
    }

    @Override
    public void printHtml() {
        start(appConf.getAppName());
        RelativePath appHtmlPath = StorageUtils.buildAppResourcePath(appConf.getAppName(), "app.html");
        DocStream docStream = multi.getResourceDocStream(appHtmlPath);
        if (docStream != null) {
            String appHtml = BdfServerUtils.toString(docStream);
            TrustedHtml trustedHtml = TrustedHtmlFactories.WELLFORMED.build(appHtml);
            this
                    .__append(trustedHtml);
        }
        JsObject jsObject = appConf.getArgsJsObject();
        if (jsObject != null) {
            this
                    .SCRIPT()
                    .__jsObject("ARGS", jsObject)
                    ._SCRIPT();
        }
        end();
    }

}
