/* FichothequeLib_API - Copyright (c) 2006-2022 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.fichotheque.corpus.metadata;

import net.fichotheque.utils.CorpusMetadataUtils;
import net.mapeadores.util.text.Labels;


/**
 *
 * @author Vincent Calame
 */
public interface CorpusField {

    public final static short NO_FICHEITEM_FIELD = 0;
    public final static short ITEM_FIELD = 1;
    public final static short PERSONNE_FIELD = 2;
    public final static short LANGUE_FIELD = 3;
    public final static short PAYS_FIELD = 4;
    public final static short DATATION_FIELD = 5;
    public final static short COURRIEL_FIELD = 6;
    public final static short LINK_FIELD = 7;
    public final static short NOMBRE_FIELD = 8;
    public final static short MONTANT_FIELD = 9;
    public final static short GEOPOINT_FIELD = 10;
    public final static short PARA_FIELD = 11;
    public final static short IMAGE_FIELD = 12;

    public FieldKey getFieldKey();

    public Labels getLabels();

    public FieldOptions getFieldOptions();

    public String[] getStringOptionNameArray();

    /**
     * Null ou non vide et sans espace blanc aux extrémités (optionValue.trim()
     * == optionValue)
     */
    public String getStringOption(String optionName);

    public CorpusMetadata getCorpusMetadata();

    /**
     * Le champ des rédacteurs retourne PERSONNE_FIELD, le champ Langue retourne
     * LANGUE_FIELD.
     *
     */
    public short getFicheItemType();

    public boolean isGenerated();

    public default String getFicheItemTypeString() {
        return CorpusMetadataUtils.ficheItemTypeToString(getFicheItemType());
    }

    public default boolean isSpecial() {
        return getFieldKey().isSpecial();
    }

    public default boolean isPropriete() {
        return getFieldKey().isPropriete();
    }

    public default boolean isInformation() {
        return getFieldKey().isInformation();
    }

    public default boolean isSection() {
        return getFieldKey().isSection();
    }

    public default String getFieldName() {
        return getFieldKey().getFieldName();
    }

    public default String getFieldString() {
        return getFieldKey().getFieldString();
    }

    public default short getCategory() {
        return getFieldKey().getCategory();
    }

    public default String getCategoryString() {
        return FieldKey.categoryToString(getFieldKey().getCategory());
    }

}
