/* FichothequeLib_Tools - Copyright (c) 2018 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.fichotheque.tools.dom;

import java.text.ParseException;
import net.fichotheque.MetadataEditor;
import net.mapeadores.util.logging.MessageHandler;
import net.mapeadores.util.text.Label;
import net.mapeadores.util.text.LabelUtils;
import net.mapeadores.util.xml.DOMUtils;
import net.mapeadores.util.xml.DomMessages;
import net.mapeadores.util.xml.ElementHandler;
import org.w3c.dom.Element;


/**
 *
 * @author Vincent Calame
 */
abstract class AbstractMetadataDOMReader {

    protected final MetadataEditor metadataEditor;
    protected final MessageHandler messageHandler;
    private final AttributeDOMReader attributeDOMReader;
    private final String oldIntituleName;

    AbstractMetadataDOMReader(MetadataEditor metadataEditor, MessageHandler messageHandler, String oldIntituleName) {
        this.metadataEditor = metadataEditor;
        this.messageHandler = messageHandler;
        this.attributeDOMReader = new AttributeDOMReader(metadataEditor.getFichothequeEditor(), messageHandler);
        this.oldIntituleName = oldIntituleName;
    }

    boolean readCommonElement(Element element, String parentXpath) {
        String tagName = element.getTagName();
        String xpath = parentXpath + "/" + tagName;
        boolean common = true;
        if (tagName.equals("label")) {
            try {
                Label label = LabelUtils.readLabel(element);
                if (label != null) {
                    metadataEditor.putLabel(null, label);
                }
            } catch (ParseException pe) {
                DomMessages.wrongLangAttribute(messageHandler, xpath, element.getAttribute("xml:lang"));
            }
        } else if (tagName.equals("phrase")) {
            String name = element.getAttribute("name");
            if (name.length() == 0) {
                DomMessages.emptyAttribute(messageHandler, xpath, "name");
            } else {
                xpath = xpath + "[@name='" + name + "']";
                PhraseLabelHandler handler = new PhraseLabelHandler(name, xpath);
                DOMUtils.readChildren(element, handler);
            }
        } else if (tagName.equals("attr")) {
            attributeDOMReader.readAttrElement(element, metadataEditor.getMetadata(), xpath);
        } else if (tagName.equals("intitule")) {
            FichothequeDOMUtils.readIntitule(element, metadataEditor, messageHandler, oldIntituleName);
        } else {
            common = false;
        }
        return common;
    }


    private class PhraseLabelHandler implements ElementHandler {

        private final String phraseName;
        private final String parentXpath;

        PhraseLabelHandler(String phraseName, String parentXpath) {
            this.phraseName = phraseName;
            this.parentXpath = parentXpath;
        }

        @Override
        public void readElement(Element element) {
            String tagName = element.getTagName();
            String xpath = parentXpath + "/" + tagName;
            if (tagName.equals("label")) {
                try {
                    Label label = LabelUtils.readLabel(element);
                    if (label != null) {
                        metadataEditor.putLabel(phraseName, label);
                    }
                } catch (ParseException pe) {
                    DomMessages.wrongLangAttribute(messageHandler, xpath, element.getAttribute("xml:lang"));
                }
            } else {
                DomMessages.unknownTagWarning(messageHandler, xpath);
            }
        }

    }

}
