/* FichothequeLib_Tools - Copyright (c) 2013-2016 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.fichotheque.tools.dom;

import net.fichotheque.album.AlbumEditor;
import net.fichotheque.album.Illustration;
import net.mapeadores.util.logging.MessageHandler;
import net.mapeadores.util.xml.DOMUtils;
import net.mapeadores.util.xml.DomMessages;
import net.mapeadores.util.xml.ElementHandler;
import org.w3c.dom.Element;


/**
 *
 * @author Vincent Calame
 */
public class IllustrationDOMReader {

    private final MessageHandler messageHandler;
    private final AlbumEditor albumEditor;
    private final AttributeDOMReader attributeDOMReader;
    private final RootHandler rootHandler = new RootHandler();

    public IllustrationDOMReader(AlbumEditor albumEditor, MessageHandler messageHandler) {
        this.albumEditor = albumEditor;
        this.messageHandler = messageHandler;
        this.attributeDOMReader = new AttributeDOMReader(albumEditor.getFichothequeEditor(), messageHandler);
    }

    public void fillIllustration(Illustration illustration, Element element) {
        rootHandler.setIllustration(illustration);
        DOMUtils.readChildren(element, rootHandler);
    }


    private class RootHandler implements ElementHandler {

        private Illustration illustration;

        private RootHandler() {
        }

        private void setIllustration(Illustration illustration) {
            this.illustration = illustration;
        }

        @Override
        public void readElement(Element element) {
            String tagName = element.getTagName();
            if (tagName.equals("attr")) {
                attributeDOMReader.readAttrElement(element, illustration, tagName);
            } else {
                DomMessages.unknownTagWarning(messageHandler, tagName);
            }
        }

    }

}
