/* FichothequeLib_Tools - Copyright (c) 2011-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.fichotheque.tools.extraction.builders;

import java.util.Collections;
import java.util.List;
import net.fichotheque.extraction.ExtractionConstants;
import net.fichotheque.extraction.def.ExtraitAlbumDef;
import net.fichotheque.extraction.def.IllustrationFilter;
import net.fichotheque.selection.IllustrationCondition;
import net.fichotheque.utils.SelectionUtils;


/**
 *
 * @author Vincent Calame
 */
public class ExtraitAlbumDefBuilder {

    private IllustrationFilter illustrationFilter;
    private String extraitName = null;
    private String tagName = ExtractionConstants.EXTRAITALBUM_DEFAULT_TAGNAME;
    private List<IllustrationCondition.Entry> entryList = SelectionUtils.EMPTY_ILLUSTRATIONCONDITIONENTRYLIST;

    public ExtraitAlbumDefBuilder(IllustrationFilter illustrationFilter) {
        if (illustrationFilter == null) {
            throw new IllegalArgumentException("illustrationFilter is null");
        }
        this.illustrationFilter = illustrationFilter;
    }


    public ExtraitAlbumDefBuilder setIllustrationFilter(IllustrationFilter illustrationFilter) {
        if (illustrationFilter == null) {
            throw new IllegalArgumentException("illustrationFilter is null");
        }
        this.illustrationFilter = illustrationFilter;
        return this;
    }

    public ExtraitAlbumDefBuilder setTagName(String tagName) {
        this.tagName = BuilderUtils.testTagName(tagName, ExtractionConstants.EXTRAITALBUM_DEFAULT_TAGNAME);
        return this;
    }

    public ExtraitAlbumDefBuilder setExtraitName(String extraitName) {
        if ((extraitName != null) && (extraitName.length() == 0)) {
            extraitName = null;
        }
        this.extraitName = extraitName;
        return this;
    }

    public ExtraitAlbumDefBuilder setEntryList(List<IllustrationCondition.Entry> list) {
        this.entryList = list;
        return this;
    }

    public ExtraitAlbumDefBuilder setEntryList(IllustrationCondition.Entry entry) {
        this.entryList = Collections.singletonList(entry);
        return this;
    }

    public ExtraitAlbumDef toExtraitAlbumDef() {
        return new InternalExtraitAlbumDef(illustrationFilter, extraitName, tagName, entryList);
    }

    public static ExtraitAlbumDefBuilder init(IllustrationFilter illustrationFilter) {
        return new ExtraitAlbumDefBuilder(illustrationFilter);
    }


    private static class InternalExtraitAlbumDef implements ExtraitAlbumDef {

        private final IllustrationFilter illustrationFilter;
        private final String extraitName;
        private final String tagName;
        private final List<IllustrationCondition.Entry> entryList;

        private InternalExtraitAlbumDef(IllustrationFilter illustrationFilter, String extraitName, String tagName, List<IllustrationCondition.Entry> entryList) {
            this.illustrationFilter = illustrationFilter;
            this.extraitName = extraitName;
            this.tagName = tagName;
            this.entryList = entryList;
        }

        @Override
        public String getTagName() {
            return tagName;
        }

        @Override
        public String getExtraitName() {
            return extraitName;
        }

        @Override
        public IllustrationFilter getIllustrationFilter() {
            return illustrationFilter;
        }

        @Override
        public List<IllustrationCondition.Entry> getEntryList() {
            return entryList;
        }

    }

}
