/* UtilLib - Copyright (c) 2007-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.conditions;

import java.io.IOException;
import net.mapeadores.util.xml.XMLUtils;
import net.mapeadores.util.xml.XMLWriter;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


/**
 *
 * @author Vincent Calame
 */
public final class ConditionsXMLStorage {

    private ConditionsXMLStorage() {
    }

    public static void appendCondition(XMLWriter xmlWriter, Condition condition) throws IOException {
        xmlWriter.startOpenTag("condition");
        xmlWriter.addAttribute("mode", ConditionsUtils.modeToString(condition.getMode()));
        xmlWriter.endOpenTag();
        for (TextTest textTest : condition.getIncludingTextTestList()) {
            appendTestText(textTest, xmlWriter);
        }
        for (TextTest textTest : condition.getExcludingTextTestList()) {
            appendTestText(textTest, xmlWriter);
        }
        xmlWriter.closeTag("condition");
    }

    public static void appendTestText(TextTest textTest, XMLWriter xmlWriter) throws IOException {
        xmlWriter.startOpenTag("text-test");
        xmlWriter.addAttribute("type", ConditionsUtils.testTypeToString(textTest.getTestType()));
        xmlWriter.endOpenTag();
        xmlWriter.addText(textTest.getText());
        xmlWriter.closeTag("text-test", false);
    }

    public static boolean isConditionElement(String tagName) {
        return (tagName.equals("condition")) || (tagName.equals("condition-group"));
    }

    public static Condition getCondition(Element element_xml) {
        String modeString = element_xml.getAttribute("mode");
        short mode = (modeString.equals("and")) ? ConditionsConstants.MODE_AND : ConditionsConstants.MODE_OR;
        String q = element_xml.getAttribute("q");
        if (q.length() == 0) {
            q = element_xml.getAttribute("raw");
        }
        if (q.length() > 0) {
            return ConditionsUtils.parseCondition(q, mode);
        }
        ConditionBuilder conditionBuilder = new ConditionBuilder(mode);
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            Node nd = liste.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element el = (Element) nd;
                String tagname = el.getTagName();
                if ((tagname.equals("text-test")) || (tagname.equals("condition"))) {
                    TextTest textTest = getTextTest(el);
                    if (textTest != null) {
                        conditionBuilder.addTextTest(textTest);
                    }
                }
            }
        }
        return conditionBuilder.toCondition();
    }

    private static TextTest getTextTest(Element element_xml) {
        String text = XMLUtils.getData(element_xml);
        String type = element_xml.getAttribute("type");
        short testType;
        if (type.length() == 0) {
            testType = ConditionsConstants.CONTAINS_TEST;
        } else {
            try {
                testType = ConditionsUtils.testTypeToShort(type);
            } catch (IllegalArgumentException iae) {
                return null;
            }
        }
        return ConditionsUtils.toTextTest(testType, text);
    }

}
