/* UtilLib - Copyright (c) 2019 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.exec;

import java.util.ArrayList;
import java.util.List;
import net.mapeadores.util.xml.DOMUtils;
import net.mapeadores.util.xml.ElementHandler;
import org.w3c.dom.Element;


/**
 *
 * @author Vincent Calame
 */
public class ArgsDefDOMReader {

    private final ArgsDefBuilder argsDefBuilder;

    public ArgsDefDOMReader(ArgsDefBuilder argsDefBuilder) {
        this.argsDefBuilder = argsDefBuilder;
    }

    public void readArgs(Element element_xml) {
        DOMUtils.readChildren(element_xml, new RootHandler());
    }

    public static void read(ArgsDefBuilder argsDefBuilder, Element rootElement) {
        ArgsDefDOMReader reader = new ArgsDefDOMReader(argsDefBuilder);
        reader.readArgs(rootElement);
    }


    private class RootHandler implements ElementHandler {

        private RootHandler() {
        }

        @Override
        public void readElement(Element element) {
            String tagName = element.getTagName();
            if (tagName.equals("parameter")) {
                String charValue = element.getAttribute("char");
                if (charValue.length() > 0) {
                    char letter = charValue.charAt(0);
                    String titleLocKey = element.getAttribute("lockey");
                    String valueState = element.getAttribute("value");
                    if (valueState.equals("require")) {
                        argsDefBuilder.addParameter(letter, titleLocKey, true);
                    } else if (valueState.equals("list")) {
                        List<String> valueList = new ArrayList<String>();
                        DOMUtils.readChildren(element, new ValueListHandler(valueList));
                        argsDefBuilder.addParameter(letter, titleLocKey, valueList);
                    } else {
                        argsDefBuilder.addParameter(letter, titleLocKey, false);
                    }
                }
            } else if (tagName.equals("command")) {
                String name = element.getAttribute("name");
                if (name.length() > 0) {
                    CommandDefBuilder commandDefBuilder = CommandDefBuilder.init(name).setTitleLocKey(element.getAttribute("lockey"));
                    DOMUtils.readChildren(element, new CommandHandler(commandDefBuilder));
                    argsDefBuilder.addCommandDef(commandDefBuilder.toCommandDef());
                }
            }
        }

    }


    private static class ValueListHandler implements ElementHandler {

        private final List<String> valueList;

        private ValueListHandler(List<String> valueList) {
            this.valueList = valueList;
        }

        @Override
        public void readElement(Element element) {
            String tagName = element.getTagName();
            if (tagName.equals("value")) {
                valueList.add(element.getAttribute("name"));
            }
        }

    }


    private static class CommandHandler implements ElementHandler {

        private final CommandDefBuilder commandDefBuilder;

        private CommandHandler(CommandDefBuilder commandDefBuilder) {
            this.commandDefBuilder = commandDefBuilder;
        }

        @Override
        public void readElement(Element element) {
            String tagName = element.getTagName();
            if (tagName.equals("mandatory")) {
                addUserParameter(element, true);
            } else if (tagName.equals("optional")) {
                addUserParameter(element, false);
            }
        }

        private void addUserParameter(Element element, boolean mandatory) {
            String value = DOMUtils.readSimpleElement(element);
            if (value.length() > 0) {
                commandDefBuilder.addUsedParameter(value.charAt(0), mandatory);
            }
        }

    }

}
